@@ -1,7 +1,7 @@
 /**************************************************************************
  *   proto.h  --  This file is part of GNU nano.                          *
  *                                                                        *
- *   Copyright (C) 1999-2011, 2013-2017 Free Software Foundation, Inc.    *
+ *   Copyright (C) 1999-2011, 2013-2018 Free Software Foundation, Inc.    *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
  *   it under the terms of the GNU General Public License as published    *
@@ -30,7 +30,7 @@ extern volatile sig_atomic_t the_window_resized;
 #endif
 
 #ifdef __linux__
-extern bool console;
+extern bool on_a_vt;
 #endif
 
 extern bool meta_key;
@@ -40,9 +40,6 @@ extern bool focusing;
 
 extern bool as_an_at;
 
-extern int margin;
-extern int editwincols;
-
 extern bool suppress_cursorpos;
 
 extern message_type lastmessage;
@@ -54,6 +51,8 @@ extern char *title;
 
 extern bool more_than_one;
 
+extern bool also_the_last;
+
 extern int didfind;
 
 extern int controlleft;
@@ -89,10 +88,13 @@ extern char *last_search;
 extern char *present_path;
 
 extern unsigned flags[4];
+
 extern WINDOW *topwin;
 extern WINDOW *edit;
 extern WINDOW *bottomwin;
 extern int editwinrows;
+extern int editwincols;
+extern int margin;
 
 extern filestruct *cutbuffer;
 extern filestruct *cutbottom;
@@ -122,6 +124,7 @@ extern char *quoteerr;
 extern char *word_chars;
 
 extern char *answer;
+extern size_t statusbar_x;
 
 extern ssize_t tabsize;
 
@@ -170,7 +173,7 @@ extern regmatch_t regmatches[10];
 
 extern int hilite_attribute;
 #ifdef ENABLE_COLOR
-extern char* specified_color_combo[NUMBER_OF_ELEMENTS];
+extern colortype *color_combo[NUMBER_OF_ELEMENTS];
 #endif
 extern int interface_color_pair[NUMBER_OF_ELEMENTS];
 
@@ -189,8 +192,8 @@ void browser_refresh(void);
 void browser_select_dirname(const char *needle);
 void do_filesearch(void);
 void do_fileresearch(bool forwards);
-void do_first_file(void);
-void do_last_file(void);
+void to_first_file(void);
+void to_last_file(void);
 char *strip_last_component(const char *path);
 #endif
 
@@ -216,10 +219,8 @@ size_t move_mbright(const char *buf, size_t pos);
 int mbstrcasecmp(const char *s1, const char *s2);
 int mbstrncasecmp(const char *s1, const char *s2, size_t n);
 char *mbstrcasestr(const char *haystack, const char *needle);
-char *revstrstr(const char *haystack, const char *needle,
-	const char *index);
-char *mbrevstrcasestr(const char *haystack, const char *needle,
-	const char *index);
+char *revstrstr(const char *haystack, const char *needle, const char *index);
+char *mbrevstrcasestr(const char *haystack, const char *needle, const char *index);
 size_t mbstrlen(const char *s);
 size_t mbstrnlen(const char *s, size_t maxlen);
 #if !defined(NANO_TINY) || defined(ENABLE_JUSTIFY)
@@ -266,12 +267,13 @@ void do_uncut_text(void);
 
 /* Most functions in files.c. */
 void initialize_buffer_text(void);
-bool open_buffer(const char *filename, bool undoable);
+void set_modified(void);
+bool open_buffer(const char *filename, bool new_buffer);
 #ifdef ENABLE_SPELLER
 void replace_buffer(const char *filename);
 #ifndef NANO_TINY
 void replace_marked_buffer(const char *filename, filestruct *top, size_t top_x,
-	filestruct *bot, size_t bot_x);
+		filestruct *bot, size_t bot_x);
 #endif
 #endif
 void prepare_for_display(void);
@@ -280,8 +282,7 @@ void switch_to_prev_buffer(void);
 void switch_to_next_buffer(void);
 bool close_buffer(void);
 #endif
-void read_file(FILE *f, int fd, const char *filename, bool undoable,
-		bool checkwritable);
+void read_file(FILE *f, int fd, const char *filename, bool undoable);
 int open_file(const char *filename, bool newfie, bool quiet, FILE **f);
 char *get_next_filename(const char *name, const char *suffix);
 void do_insertfile_void(void);
@@ -298,10 +299,10 @@ int write_lockfile(const char *lockfilename, const char *origfilename, bool modi
 #endif
 int copy_file(FILE *inn, FILE *out, bool close_out);
 bool write_file(const char *name, FILE *f_open, bool tmp,
-	kind_of_writing_type method, bool nonamechange);
+		kind_of_writing_type method, bool fullbuffer);
 #ifndef NANO_TINY
 bool write_marked_file(const char *name, FILE *f_open, bool tmp,
-	kind_of_writing_type method);
+		kind_of_writing_type method);
 #endif
 int do_writeout(bool exiting, bool withprompt);
 void do_writeout_void(void);
@@ -312,7 +313,7 @@ int diralphasort(const void *va, const void *vb);
 #endif
 #ifdef ENABLE_TABCOMP
 char *input_tab(char *buf, bool allow_files, size_t *place,
-	bool *lastwastab, void (*refresh_func)(void), bool *listed);
+		bool *lastwastab, void (*refresh_func)(void), bool *listed);
 #endif
 
 /* Some functions in global.c. */
@@ -367,8 +368,8 @@ bool has_old_position(const char *file, ssize_t *line, ssize_t *column);
 #endif
 
 /* Most functions in move.c. */
-void do_first_line(void);
-void do_last_line(void);
+void to_first_line(void);
+void to_last_line(void);
 void do_page_up(void);
 void do_page_down(void);
 #ifdef ENABLE_JUSTIFY
@@ -380,16 +381,14 @@ void do_para_end_void(void);
 void do_prev_block(void);
 void do_next_block(void);
 void do_prev_word(bool allow_punct, bool update_screen);
-bool do_next_word(bool allow_punct, bool update_screen);
+bool do_next_word(bool after_ends, bool allow_punct, bool update_screen);
 void do_prev_word_void(void);
 void do_next_word_void(void);
 void do_home(void);
 void do_end(void);
-void do_up(bool scroll_only);
-void do_down(bool scroll_only);
-void do_up_void(void);
-void do_down_void(void);
-#ifndef NANO_TINY
+void do_up(void);
+void do_down(void);
+#ifdef ENABLE_HELP
 void do_scroll_up(void);
 void do_scroll_down(void);
 #endif
@@ -403,17 +402,19 @@ void unlink_node(filestruct *fileptr);
 void delete_node(filestruct *fileptr);
 filestruct *copy_filestruct(const filestruct *src);
 void free_filestruct(filestruct *src);
-void renumber(filestruct *fileptr);
+void renumber(filestruct *line);
 partition *partition_filestruct(filestruct *top, size_t top_x,
-	filestruct *bot, size_t bot_x);
+		filestruct *bot, size_t bot_x);
 void unpartition_filestruct(partition **p);
 void extract_buffer(filestruct **file_top, filestruct **file_bot,
-	filestruct *top, size_t top_x, filestruct *bot, size_t bot_x);
+		filestruct *top, size_t top_x, filestruct *bot, size_t bot_x);
 void ingraft_buffer(filestruct *somebuffer);
 void copy_from_buffer(filestruct *somebuffer);
 openfilestruct *make_new_opennode(void);
+#ifdef ENABLE_MULTIBUFFER
 void unlink_opennode(openfilestruct *fileptr);
 void delete_opennode(openfilestruct *fileptr);
+#endif
 void print_view_warning(void);
 void show_restricted_warning(void);
 #ifndef ENABLE_HELP
@@ -421,11 +422,14 @@ void say_there_is_no_help(void);
 #endif
 void finish(void);
 void die(const char *msg, ...);
-void die_save_file(const char *die_filename, struct stat *die_stat);
+void emergency_save(const char *die_filename, struct stat *die_stat);
 void window_init(void);
 void do_exit(void);
 void close_and_go(void);
 RETSIGTYPE handle_hupterm(int signal);
+#ifndef DEBUG
+RETSIGTYPE handle_crash(int signal);
+#endif
 RETSIGTYPE do_suspend(int signal);
 RETSIGTYPE do_continue(int signal);
 #ifndef NANO_TINY
@@ -433,25 +437,21 @@ RETSIGTYPE handle_sigwinch(int signal);
 void regenerate_screen(void);
 void allow_sigwinch(bool allow);
 void do_toggle(int flag);
-void do_toggle_void(void);
 void enable_signals(void);
 #endif
 void disable_flow_control(void);
 void enable_flow_control(void);
 void terminal_init(void);
-void unbound_key(int code);
-int do_input(bool allow_funcs);
-#ifdef ENABLE_MOUSE
-int do_mouse(void);
+#ifdef ENABLE_LINENUMBERS
+void confirm_margin(void);
 #endif
+void unbound_key(int code);
+bool okay_for_view(const sc *shortcut);
+int do_input(bool allow_funcs);
 void do_output(char *output, size_t output_len, bool allow_cntrls);
 
 /* Most functions in prompt.c. */
-#ifdef ENABLE_MOUSE
-int do_statusbar_mouse(void);
-#endif
-void do_statusbar_output(int *the_input, size_t input_len,
-	bool filtering);
+void do_statusbar_output(int *the_input, size_t input_len, bool filtering);
 void do_statusbar_home(void);
 void do_statusbar_end(void);
 void do_statusbar_left(void);
@@ -470,14 +470,13 @@ size_t get_statusbar_page_start(size_t start_col, size_t column);
 void reinit_statusbar_x(void);
 void update_the_statusbar(void);
 int do_prompt(bool allow_tabs, bool allow_files,
-	int menu, const char *curranswer, filestruct **history_list,
-	void (*refresh_func)(void), const char *msg, ...);
+		int menu, const char *curranswer, filestruct **history_list,
+		void (*refresh_func)(void), const char *msg, ...);
 int do_yesno_prompt(bool all, const char *msg);
 
 /* Most functions in rcfile.c. */
 #ifdef ENABLE_NANORC
 #ifdef ENABLE_COLOR
-bool parse_color_names(char *combostr, short *fg, short *bg, bool *bright);
 void grab_and_store(const char *kind, char *ptr, regexlisttype **storage);
 #endif
 void parse_rcfile(FILE *rcstream, bool syntax_only);
@@ -485,25 +484,26 @@ void do_rcfiles(void);
 #endif /* ENABLE_NANORC */
 
 /* Most functions in search.c. */
-void not_found_msg(const char *str);
-void search_replace_abort(void);
+void tidy_up_after_search(void);
 int findnextstr(const char *needle, bool whole_word_only, int modus,
-	size_t *match_len, bool skipone, const filestruct *begin, size_t begin_x);
+		size_t *match_len, bool skipone, const filestruct *begin, size_t begin_x);
 void do_search(void);
 void do_search_forward(void);
 void do_search_backward(void);
+void do_research(void);
 #ifndef NANO_TINY
 void do_findprevious(void);
 void do_findnext(void);
 #endif
-void do_research(void);
+void not_found_msg(const char *str);
 void go_looking(void);
 ssize_t do_replace_loop(const char *needle, bool whole_word_only,
-	const filestruct *real_current, size_t *real_current_x);
+		const filestruct *real_current, size_t *real_current_x);
 void do_replace(void);
+void ask_for_replacement(void);
 void goto_line_posx(ssize_t line, size_t pos_x);
 void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
-	bool interactive);
+		bool interactive);
 void do_gotolinecolumn_void(void);
 #ifndef NANO_TINY
 void do_find_bracket(void);
@@ -527,7 +527,6 @@ void do_unindent(void);
 bool white_string(const char *s);
 #ifdef ENABLE_COMMENT
 void do_comment(void);
-bool comment_line(undo_type action, filestruct *f, const char *comment_seq);
 #endif
 void do_undo(void);
 void do_redo(void);
@@ -552,7 +551,7 @@ size_t indent_length(const char *line);
 #endif
 #ifdef ENABLE_JUSTIFY
 void justify_format(filestruct *paragraph, size_t skip);
-bool begpar(const filestruct *const foo);
+bool begpar(const filestruct *const foo, int depth);
 bool inpar(const filestruct *const foo);
 void do_justify(bool full_justify);
 void do_justify_void(void);
@@ -592,7 +591,7 @@ const char *fixbounds(const char *r);
 bool is_separate_word(size_t position, size_t length, const char *buf);
 #endif
 const char *strstrwrapper(const char *haystack, const char *needle,
-	const char *start);
+		const char *start);
 void nperror(const char *s);
 void *nmalloc(size_t howmuch);
 void *nrealloc(void *ptr, size_t howmuch);
@@ -609,8 +608,9 @@ void new_magicline(void);
 void remove_magicline(void);
 #endif
 #ifndef NANO_TINY
-void mark_order(const filestruct **top, size_t *top_x, const filestruct
-	**bot, size_t *bot_x, bool *right_side_up);
+void mark_order(const filestruct **top, size_t *top_x,
+		const filestruct **bot, size_t *bot_x, bool *right_side_up);
+void get_range(const filestruct **top, const filestruct **bot);
 #endif
 size_t get_totsize(const filestruct *begin, const filestruct *end);
 #ifndef NANO_TINY
@@ -618,15 +618,17 @@ filestruct *fsfromline(ssize_t lineno);
 #endif
 #ifdef DEBUG
 void dump_filestruct(const filestruct *inptr);
-void dump_filestruct_reverse(void);
 #endif
 
 /* Most functions in winio.c. */
 void record_macro(void);
 void run_macro(void);
-void get_key_buffer(WINDOW *win);
 size_t get_key_buffer_len(void);
+void put_back(int keycode);
 void unget_kbinput(int kbinput, bool metakey);
+#ifdef ENABLE_NANORC
+void implant(const char *string);
+#endif
 int get_kbinput(WINDOW *win, bool showcursor);
 int parse_kbinput(WINDOW *win);
 int arrow_from_abcd(int kbinput);
@@ -636,17 +638,17 @@ int get_control_kbinput(int kbinput);
 int *get_verbatim_kbinput(WINDOW *win, size_t *kbinput_len);
 int *parse_verbatim_kbinput(WINDOW *win, size_t *count);
 #ifdef ENABLE_MOUSE
-int get_mouseinput(int *mouse_x, int *mouse_y, bool allow_shortcuts);
+int get_mouseinput(int *mouse_row, int *mouse_col, bool allow_shortcuts);
 #endif
 const sc *get_shortcut(int *kbinput);
 void blank_row(WINDOW *win, int y, int x, int n);
 void blank_edit(void);
 void blank_statusbar(void);
+void wipe_statusbar(void);
 void blank_bottombars(void);
 void check_statusblank(void);
 char *display_string(const char *buf, size_t column, size_t span, bool isdata);
 void titlebar(const char *path);
-extern void set_modified(void);
 void statusbar(const char *msg);
 void warn_and_shortly_pause(const char *msg);
 void statusline(message_type importance, const char *msg, ...);
@@ -654,7 +656,7 @@ void bottombars(int menu);
 void post_one_key(const char *keystroke, const char *tag, int width);
 void place_the_cursor(void);
 void edit_draw(filestruct *fileptr, const char *converted,
-	int line, size_t from_col);
+		int line, size_t from_col);
 int update_line(filestruct *fileptr, size_t index);
 #ifndef NANO_TINY
 int update_softwrapped_line(filestruct *fileptr);
@@ -663,10 +665,10 @@ bool line_needs_update(const size_t old_column, const size_t new_column);
 int go_back_chunks(int nrows, filestruct **line, size_t *leftedge);
 int go_forward_chunks(int nrows, filestruct **line, size_t *leftedge);
 bool less_than_a_screenful(size_t was_lineno, size_t was_leftedge);
-void edit_scroll(bool direction, int nrows);
+void edit_scroll(bool direction);
 #ifndef NANO_TINY
 size_t get_softwrap_breakpoint(const char *text, size_t leftedge,
-				bool *end_of_line);
+								bool *end_of_line);
 size_t get_chunk_and_edge(size_t column, filestruct *line, size_t *leftedge);
 size_t chunk_for(size_t column, filestruct *line);
 size_t leftedge_for(size_t column, filestruct *line);
@@ -694,27 +696,29 @@ void do_credits(void);
 #endif
 
 /* These are just name definitions. */
-void do_cancel(void);
 void case_sens_void(void);
 void regexp_void(void);
 void backwards_void(void);
 void flip_replace(void);
-void gototext_void(void);
+void flip_goto(void);
 #ifdef ENABLE_BROWSER
 void to_files_void(void);
 void goto_dir_void(void);
 #endif
 #ifndef NANO_TINY
+void do_toggle_void(void);
 void dos_format_void(void);
 void mac_format_void(void);
 void append_void(void);
 void prepend_void(void);
 void backup_file_void(void);
 void flip_execute(void);
+void flip_pipe(void);
 #endif
 #ifdef ENABLE_MULTIBUFFER
 void flip_newbuffer(void);
 #endif
 void discard_buffer(void);
+void do_cancel(void);
 
 #endif /* !PROTO_H */
